clear all; close all; clc;

%%  System Data
Jm=1.5e-4; %[Kgm^2]
n=100; %n=w_l/w_m
Dm=0.0034; %[Nm/(rad/s)]
Jl=2.7; %[Kgm^2]
Kel=3.05; %[Nm/rad]
Del=0.0022; %[Nm/(rad/s)]

Jlr=Jl/n^2; %[Kgm^2]

% state space system matrices:
% x=[qm;dqm;ql;dql]
A=[0,1,0,0;
    -Kel/Jm,-(Del+Dm)/Jm,Kel/Jm,Del/Jm;
    0,0,0,1;
    Kel/Jlr,Del/Jlr,-Kel/Jlr,-Del/Jlr];

B=[0;1/Jm;0;0];

%% Exercise 1

% Gvm=dq_m(s)/tau_m(s)
Gvm=tf(ss(A,B,[0,1,0,0],0))

% check:
Ccheck=[0,1,0,0];
s=tf('s'); 
Gvm_check=Ccheck*inv(s*eye(4)-A)*B

% check TF analytic 
J=Jlr+Jm;
Gvm_analytic=(Jlr*s^2+Del*s+Kel)/(Jlr*Jm*s^3+(J*Del+Jlr*Dm)*s^2+(J*Kel+Dm*Del)*s+Dm*Kel)

% zeros frequency
wz=sqrt(Kel/Jlr) %[rad/s]
% zeros damping
csiz=Del/2/sqrt(Jlr*Kel)


% poles frequency
rho=Jlr/Jm
wp=wz*sqrt(1+rho)
% poles damping
csip=csiz*sqrt(1+rho)

%% Exercise 2

% % compensator design by usual commands (annoying...)
% s=tf('s');%laplace variable
% Tiv=10/wz; %[s]
% Rv=(1+s*Tiv)/(s*Tiv); %PI regulator with Kpv=1
% Lv=Rv*Gvm %open loop velocity TF
% rlocus(Lv);sgrid;
% [Kpv,poles]=rlocfind(Lv)
% mu=1/(Jm+Jlr);
% wcv=Kpv*mu; %[rad/s] rigid model crossover freq
% wtildecv=wcv/wz %note it is almost equal to 0.7 (max damping)

%compensator design by rltool GUI (more handy)
rltool(Gvm)


% rltool(Gvm,Rv) %with predefined compensator structure

Kiv=0.33; %da rltool
Tiv=10/wz;
Kpv=Kiv*Tiv

mu=1/(Jm+Jlr);
wcv=Kpv*mu;
wtildecv=wcv/wz

%% Exercise 3
Rv=(1+s*Tiv)/(s*Tiv)
Lv=Kpv*Rv*Gvm; 
Fv=feedback(Lv,1)
Gpm=Fv/s %motor-side position controller, open loop TF

%% Exercise 4

%position compensator design with rltool "app"
rltool(Gpm)
Kpp=29.7; %from rltool

% %position compensator design with traditional commands
% rlocus(Gpm);sgrid;
% [Kpp,poles]=rlocfind(Gpm)
% Lp=Kpp*Gpm;
% Gp=feedback(Lp,1);
% damp(Gp)


%% Exercise 5
figure('name','Motor-side');

subplot(3,1,1) %qm,ql plot
plot(t,qm,t,ql,'linewidth',2);
grid on;
xlabel('t[s]');
ylabel('[rad]');
title('Motor and Load positions');
h=legend('q_m','q_l');
set(h,'FontSize',16);

subplot(3,1,2) %dqm,dql plot
plot(t,dqm,t,dql,'linewidth',2);
grid on;
xlabel('t[s]');
ylabel('[rad/s]');
title('Motor and Load velocities');
h=legend('dq_m','dq_l');
set(h,'FontSize',16);

subplot(3,1,3) %tau_m plot
plot(t,taum,'r','linewidth',2);
grid on;
xlabel('t[s]');
ylabel('[Nm]');
title('Motor Torque');

%% Exercise 6

Glm=(1+2*csiz/wz*s)/(1+2*csiz/wz*s+s^2/wz^2) %TF from qm to ql

Gpl = Fv/s*Glm %open loop TF with load-side position controller

%% Exercise 7

rltool(Gpl)

Kpp=35.34; %da rltool

%% Exercise 8

figure('name','Load-side');

subplot(3,1,1) %qm,ql plot
plot(t,qm,t,ql,'linewidth',2);
grid on;
xlabel('t[s]');
ylabel('[rad]');
title('Motor and Load positions');
h=legend('q_m','q_l');
set(h,'FontSize',16);

subplot(3,1,2) %dqm,dql plot
plot(t,dqm,t,dql,'linewidth',2);
grid on;
xlabel('t[s]');
ylabel('[rad/s]');
title('Motor and Load velocities');
h=legend('dq_m','dq_l');
set(h,'FontSize',16);

subplot(3,1,3) %tau_m plot
plot(t,taum,'r','linewidth',2);
grid on;
xlabel('t[s]');
ylabel('[Nm]');
title('Motor Torque');


